from json import dumps, loads
from tfw.crypto import KeyManager, sign_message, verify_message
from tfw.components import PipeIOEventHandlerBase
from tfw.components.pipe_io_event_handler import DEFAULT_PERMISSIONS

class DeployPipeIOEventHandler(PipeIOEventHandlerBase):
    # pylint: disable=too-many-arguments
    def __init__(
            self, in_pipe_path, out_pipe_path, process,
            permissions=DEFAULT_PERMISSIONS
    ):
        self.expected = False
        self.process = process

        self.onsuccess = {
            'key': 'processmanager',
            'data': {
                'process_name': process,
                'command': 'restart'
            }
        }
        self.onerror = {
            'key': 'processmanager',
            'data': {
                'process_name': process,
                'error': True
            }
        }

        super().__init__('processmanager', in_pipe_path, out_pipe_path, permissions)

    def handle_event(self, message):
        if message == self.onsuccess:
            self.expected = True
            self.pipe_io.send_message(b'deploy')

    def handle_pipe_event(self, message_bytes):
        if not self.expected:
            raise ValueError(
                f'{self.pipe_io.in_pipe}: There is nothing to deploy.'
            )

        self.expected = False
        if message_bytes == b'true':
            self.server_connector.send(self.onsuccess)
        elif message_bytes == b'false':
            self.server_connector.send(self.onerror)
        else:
            raise ValueError(
                f'{self.pipe_io.in_pipe}: Expected "true" or "false".'
            )

class IdePipeIOEventHandler(PipeIOEventHandlerBase):
    # pylint: disable=too-many-arguments
    def __init__(
            self, in_pipe_path, out_pipe_path, filename,
            permissions=DEFAULT_PERMISSIONS,
            selected=True
    ):
        self.selected = selected
        self.filename = filename
        super().__init__('ide', in_pipe_path, out_pipe_path, permissions)

    def handle_event(self, message):
        data = message['data']

        if data['command'] == 'select':
            self.selected = data['filename'] == self.filename
        elif data['command'] == 'write' and self.selected:
            clean = data['content'].replace('\n', '\\n')
            self.pipe_io.send_message(clean.encode())

    def handle_pipe_event(self, message_bytes):
        if not self.selected:
            self.server_connector.send({
                'key': 'mirror',
                'data': {
                    'key': 'ide',
                    'data': {
                        'command': 'select',
                        'filename': self.filename
                    }
                }
            })

        self.server_connector.send({
            'key': 'mirror',
            'data': {
                'key': 'ide',
                'data': {
                    'command': 'write',
                    'content': message_bytes.decode().replace('\\n', '\n')
                }
            }
        })
        self.server_connector.send({
            'key': 'mirror',
            'data': {
                'key': 'ide',
                'data': {
                    'command': 'read'
                }
            }
        })

class FSMPipeIOEventHandler(PipeIOEventHandlerBase):
    def __init__(self, in_pipe_path, out_pipe_path, permissions=DEFAULT_PERMISSIONS):
        super().__init__(
            ['fsm', 'fsm_update'],
            in_pipe_path, out_pipe_path, permissions
        )

    def handle_event(self, message):
        if 'current_state' in message['data']:
            self.pipe_io.send_message(message['data']['current_state'].encode())

    def handle_pipe_event(self, message_bytes):
        self.server_connector.send({
            'key': '',
            'trigger': message_bytes.decode()
        })

class MsgSignPipeIOEventHandler(PipeIOEventHandlerBase):
    def __init__(
            self, in_pipe_path, out_pipe_path,
            permissions=DEFAULT_PERMISSIONS,
            forwarding=True
    ):
        self.forwarding = forwarding
        self.auth_key = KeyManager().auth_key
        super().__init__(None, in_pipe_path, out_pipe_path, permissions)

    def handle_event(self, message):
        return

    def handle_pipe_event(self, message_bytes):
        message = loads(message_bytes)
        sign_message(self.auth_key, message)
        if self.forwarding:
            self.server_connector.send(message)
        self.pipe_io.send_message(dumps(message).encode())

class MsgVerifyPipeIOEventHandler(PipeIOEventHandlerBase):
    def __init__(self, in_pipe_path, out_pipe_path, permissions=DEFAULT_PERMISSIONS):
        self.auth_key = KeyManager().auth_key
        super().__init__(None, in_pipe_path, out_pipe_path, permissions)

    def handle_event(self, message):
        return

    def handle_pipe_event(self, message_bytes):
        message = loads(message_bytes)
        validity = verify_message(self.auth_key, message)
        self.pipe_io.send_message(str(validity).lower().encode())
