from signal import signal, SIGTERM, SIGINT

from tornado.ioloop import IOLoop

from tfw import EventHandlerBase
from tfw.components import PipeIOEventHandler

from pipe_io_auxlib import (
        MsgSignPipeIOEventHandler, MsgVerifyPipeIOEventHandler,
        DeployPipeIOEventHandler, IdePipeIOEventHandler,
        FSMPipeIOEventHandler
)


if __name__ == '__main__':
    '''
    Creates general purpose pipes.
    You can send/receive JSON messages to/from the TFW server as any user.
    '''
    json_pipe = PipeIOEventHandler(
        '',
        '/tmp/tfw_json_send',
        '/tmp/tfw_json_recv',
        permissions=0o666
    )

    '''
    Signs a valid TFW message with HMAC.
    Note that this process needs root permissions in order to read the
    authentication key.
    When forwarding is true, it will send the signed message to the TFW
    server before writing it into the output pipe.
    '''
    sign_pipe = MsgSignPipeIOEventHandler(
        '/tmp/tfw_sign_send',
        '/tmp/tfw_sign_recv',
        forwarding=True
    )

    '''
    Verifies a signed TFW message.
    This pipe also needs root permissions. Send the serialized JSON object
    to the pipe, then wait for its boolean response.
    '''
    verify_pipe = MsgVerifyPipeIOEventHandler(
        '/tmp/tfw_verify_send',
        '/tmp/tfw_verify_recv'
    )

    '''
    Manages deployment in the IDE.
    When you receive "deploy", then you have to answer with a "true" or
    "false" depending whether you are satisfied with the result or not.
    The third parameter is the name of the supervised service.
    '''
    deploy_pipe = DeployPipeIOEventHandler(
        '/tmp/tfw_deploy_send',
        '/tmp/tfw_deploy_recv',
        'webservice'
    )

    '''
    Manipulates the content of the IDE.
    You can observe a file, and when the user edits it, you will receive
    the new contents where newlines are escaped as "\\n".
    In order to overwrite the file, send an escaped text back to the pipe.
    Since the pipe doesn't know if the file is selected initially in the IDE,
    you have to provide this information by yourself, but it will track it
    later on.
    '''
    ide_pipe = IdePipeIOEventHandler(
        '/tmp/tfw_ide_send',
        '/tmp/tfw_ide_recv',
        'user_ops.py',
        selected=True
    )

    '''
    Handles FSM steps.
    When the FSM enters the next state, you will receive a line containing
    its name.
    To trigger a state change, send the name of the transition to the pipe.
    '''
    fsm_pipe = FSMPipeIOEventHandler(
        '/tmp/tfw_fsm_send',
        '/tmp/tfw_fsm_recv'
    )

    event_handlers = EventHandlerBase.get_local_instances()
    def cleanup(sig, frame):
        for eh in event_handlers:
            eh.cleanup()
        exit(0)
    signal(SIGTERM, cleanup)
    signal(SIGINT, cleanup)

    IOLoop.instance().start()
